@@ -37,21 +37,28 @@
 #include "err.hpp"
 #include "curve_server.hpp"
 #include "wire.hpp"
+#include "secure_allocator.hpp"
 
 zmq::curve_server_t::curve_server_t (session_base_t *session_,
                                      const std::string &peer_address_,
-                                     const options_t &options_) :
+                                     const options_t &options_,
+                                     const bool downgrade_sub_) :
     mechanism_base_t (session_, options_),
     zap_client_common_handshake_t (
       session_, peer_address_, options_, sending_ready),
-    curve_mechanism_base_t (
-      session_, options_, "CurveZMQMESSAGES", "CurveZMQMESSAGEC")
+    curve_mechanism_base_t (session_,
+                            options_,
+                            "CurveZMQMESSAGES",
+                            "CurveZMQMESSAGEC",
+                            downgrade_sub_)
 {
     int rc;
     //  Fetch our secret key from socket options
     memcpy (_secret_key, options_.curve_secret_key, crypto_box_SECRETKEYBYTES);
 
     //  Generate short-term key pair
+    memset (_cn_secret, 0, crypto_box_SECRETKEYBYTES);
+    memset (_cn_public, 0, crypto_box_PUBLICKEYBYTES);
     rc = crypto_box_keypair (_cn_public, _cn_secret);
     zmq_assert (rc == 0);
 }
@@ -174,18 +181,19 @@ int zmq::curve_server_t::process_hello (msg_t *msg_)
     memcpy (_cn_client, hello + 80, 32);
 
     uint8_t hello_nonce[crypto_box_NONCEBYTES];
-    uint8_t hello_plaintext[crypto_box_ZEROBYTES + 64];
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > hello_plaintext (
+      crypto_box_ZEROBYTES + 64);
     uint8_t hello_box[crypto_box_BOXZEROBYTES + 80];
 
     memcpy (hello_nonce, "CurveZMQHELLO---", 16);
     memcpy (hello_nonce + 16, hello + 112, 8);
-    cn_peer_nonce = get_uint64 (hello + 112);
+    set_peer_nonce (get_uint64 (hello + 112));
 
     memset (hello_box, 0, crypto_box_BOXZEROBYTES);
     memcpy (hello_box + crypto_box_BOXZEROBYTES, hello + 120, 80);
 
     //  Open Box [64 * %x0](C'->S)
-    rc = crypto_box_open (hello_plaintext, hello_box, sizeof hello_box,
+    rc = crypto_box_open (&hello_plaintext[0], hello_box, sizeof hello_box,
                           hello_nonce, _cn_client, _secret_key);
     if (rc != 0) {
         // CURVE I: cannot open client HELLO -- wrong server key?
@@ -202,47 +210,54 @@ int zmq::curve_server_t::process_hello (msg_t *msg_)
 int zmq::curve_server_t::produce_welcome (msg_t *msg_)
 {
     uint8_t cookie_nonce[crypto_secretbox_NONCEBYTES];
-    uint8_t cookie_plaintext[crypto_secretbox_ZEROBYTES + 64];
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > cookie_plaintext (
+      crypto_secretbox_ZEROBYTES + 64);
     uint8_t cookie_ciphertext[crypto_secretbox_BOXZEROBYTES + 80];
 
     //  Create full nonce for encryption
     //  8-byte prefix plus 16-byte random nonce
+    memset (cookie_nonce, 0, crypto_secretbox_NONCEBYTES);
     memcpy (cookie_nonce, "COOKIE--", 8);
     randombytes (cookie_nonce + 8, 16);
 
     //  Generate cookie = Box [C' + s'](t)
-    memset (cookie_plaintext, 0, crypto_secretbox_ZEROBYTES);
-    memcpy (cookie_plaintext + crypto_secretbox_ZEROBYTES, _cn_client, 32);
-    memcpy (cookie_plaintext + crypto_secretbox_ZEROBYTES + 32, _cn_secret, 32);
+    std::fill (cookie_plaintext.begin (),
+               cookie_plaintext.begin () + crypto_secretbox_ZEROBYTES, 0);
+    memcpy (&cookie_plaintext[crypto_secretbox_ZEROBYTES], _cn_client, 32);
+    memcpy (&cookie_plaintext[crypto_secretbox_ZEROBYTES + 32], _cn_secret, 32);
 
     //  Generate fresh cookie key
+    memset (_cookie_key, 0, crypto_secretbox_KEYBYTES);
     randombytes (_cookie_key, crypto_secretbox_KEYBYTES);
 
     //  Encrypt using symmetric cookie key
     int rc =
-      crypto_secretbox (cookie_ciphertext, cookie_plaintext,
-                        sizeof cookie_plaintext, cookie_nonce, _cookie_key);
+      crypto_secretbox (cookie_ciphertext, &cookie_plaintext[0],
+                        cookie_plaintext.size (), cookie_nonce, _cookie_key);
     zmq_assert (rc == 0);
 
     uint8_t welcome_nonce[crypto_box_NONCEBYTES];
-    uint8_t welcome_plaintext[crypto_box_ZEROBYTES + 128];
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > welcome_plaintext (
+      crypto_box_ZEROBYTES + 128);
     uint8_t welcome_ciphertext[crypto_box_BOXZEROBYTES + 144];
 
     //  Create full nonce for encryption
     //  8-byte prefix plus 16-byte random nonce
+    memset (welcome_nonce, 0, crypto_box_NONCEBYTES);
     memcpy (welcome_nonce, "WELCOME-", 8);
     randombytes (welcome_nonce + 8, crypto_box_NONCEBYTES - 8);
 
     //  Create 144-byte Box [S' + cookie](S->C')
-    memset (welcome_plaintext, 0, crypto_box_ZEROBYTES);
-    memcpy (welcome_plaintext + crypto_box_ZEROBYTES, _cn_public, 32);
-    memcpy (welcome_plaintext + crypto_box_ZEROBYTES + 32, cookie_nonce + 8,
+    std::fill (welcome_plaintext.begin (),
+               welcome_plaintext.begin () + crypto_box_ZEROBYTES, 0);
+    memcpy (&welcome_plaintext[crypto_box_ZEROBYTES], _cn_public, 32);
+    memcpy (&welcome_plaintext[crypto_box_ZEROBYTES + 32], cookie_nonce + 8,
             16);
-    memcpy (welcome_plaintext + crypto_box_ZEROBYTES + 48,
+    memcpy (&welcome_plaintext[crypto_box_ZEROBYTES + 48],
             cookie_ciphertext + crypto_secretbox_BOXZEROBYTES, 80);
 
-    rc = crypto_box (welcome_ciphertext, welcome_plaintext,
-                     sizeof welcome_plaintext, welcome_nonce, _cn_client,
+    rc = crypto_box (welcome_ciphertext, &welcome_plaintext[0],
+                     welcome_plaintext.size (), welcome_nonce, _cn_client,
                      _secret_key);
 
     //  TODO I think we should change this back to zmq_assert (rc == 0);
@@ -327,61 +342,59 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
     const size_t clen = (size - 113) + crypto_box_BOXZEROBYTES;
 
     uint8_t initiate_nonce[crypto_box_NONCEBYTES];
-    uint8_t *initiate_plaintext =
-      static_cast<uint8_t *> (malloc (crypto_box_ZEROBYTES + clen));
-    alloc_assert (initiate_plaintext);
-    uint8_t *initiate_box =
-      static_cast<uint8_t *> (malloc (crypto_box_BOXZEROBYTES + clen));
-    alloc_assert (initiate_box);
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > initiate_plaintext (
+      crypto_box_ZEROBYTES + clen);
+    std::vector<uint8_t> initiate_box (crypto_box_BOXZEROBYTES + clen);
 
     //  Open Box [C + vouch + metadata](C'->S')
-    memset (initiate_box, 0, crypto_box_BOXZEROBYTES);
-    memcpy (initiate_box + crypto_box_BOXZEROBYTES, initiate + 113,
+    std::fill (initiate_box.begin (),
+               initiate_box.begin () + crypto_box_BOXZEROBYTES, 0);
+    memcpy (&initiate_box[crypto_box_BOXZEROBYTES], initiate + 113,
             clen - crypto_box_BOXZEROBYTES);
 
     memcpy (initiate_nonce, "CurveZMQINITIATE", 16);
     memcpy (initiate_nonce + 16, initiate + 105, 8);
-    cn_peer_nonce = get_uint64 (initiate + 105);
+    set_peer_nonce (get_uint64 (initiate + 105));
 
-    const uint8_t *client_key = initiate_plaintext + crypto_box_ZEROBYTES;
+    const uint8_t *client_key = &initiate_plaintext[crypto_box_ZEROBYTES];
 
-    rc = crypto_box_open (initiate_plaintext, initiate_box, clen,
+    rc = crypto_box_open (&initiate_plaintext[0], &initiate_box[0], clen,
                           initiate_nonce, _cn_client, _cn_secret);
     if (rc != 0) {
         // CURVE I: cannot open client INITIATE
         session->get_socket ()->event_handshake_failed_protocol (
           session->get_endpoint (), ZMQ_PROTOCOL_ERROR_ZMTP_CRYPTOGRAPHIC);
         errno = EPROTO;
-        rc = -1;
-        goto exit;
+        return -1;
     }
 
     uint8_t vouch_nonce[crypto_box_NONCEBYTES];
-    uint8_t vouch_plaintext[crypto_box_ZEROBYTES + 64];
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > vouch_plaintext (
+      crypto_box_ZEROBYTES + 64);
     uint8_t vouch_box[crypto_box_BOXZEROBYTES + 80];
 
     //  Open Box Box [C',S](C->S') and check contents
     memset (vouch_box, 0, crypto_box_BOXZEROBYTES);
     memcpy (vouch_box + crypto_box_BOXZEROBYTES,
-            initiate_plaintext + crypto_box_ZEROBYTES + 48, 80);
+            &initiate_plaintext[crypto_box_ZEROBYTES + 48], 80);
 
+    memset (vouch_nonce, 0, crypto_box_NONCEBYTES);
     memcpy (vouch_nonce, "VOUCH---", 8);
-    memcpy (vouch_nonce + 8, initiate_plaintext + crypto_box_ZEROBYTES + 32,
+    memcpy (vouch_nonce + 8, &initiate_plaintext[crypto_box_ZEROBYTES + 32],
             16);
 
-    rc = crypto_box_open (vouch_plaintext, vouch_box, sizeof vouch_box,
+    rc = crypto_box_open (&vouch_plaintext[0], vouch_box, sizeof vouch_box,
                           vouch_nonce, client_key, _cn_secret);
     if (rc != 0) {
         // CURVE I: cannot open client INITIATE vouch
         session->get_socket ()->event_handshake_failed_protocol (
           session->get_endpoint (), ZMQ_PROTOCOL_ERROR_ZMTP_CRYPTOGRAPHIC);
         errno = EPROTO;
-        rc = -1;
-        goto exit;
+        return -1;
     }
 
     //  What we decrypted must be the client's short-term public key
-    if (memcmp (vouch_plaintext + crypto_box_ZEROBYTES, _cn_client, 32)) {
+    if (memcmp (&vouch_plaintext[crypto_box_ZEROBYTES], _cn_client, 32)) {
         // TODO this case is very hard to test, as it would require a modified
         //  client that knows the server's secret short-term key
 
@@ -389,12 +402,12 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
         session->get_socket ()->event_handshake_failed_protocol (
           session->get_endpoint (), ZMQ_PROTOCOL_ERROR_ZMTP_KEY_EXCHANGE);
         errno = EPROTO;
-        rc = -1;
-        goto exit;
+        return -1;
     }
 
     //  Precompute connection secret from client key
-    rc = crypto_box_beforenm (cn_precom, _cn_client, _cn_secret);
+    rc = crypto_box_beforenm (get_writable_precom_buffer (), _cn_client,
+                              _cn_secret);
     zmq_assert (rc == 0);
 
     //  Given this is a backward-incompatible change, it's behind a socket
@@ -410,9 +423,8 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
             //  reply already, but removing this has some strange side-effect
             //  (probably because the pipe's in_active flag is true until a read
             //  is attempted)
-            rc = receive_and_process_zap_reply ();
-            if (rc == -1)
-                goto exit;
+            if (-1 == receive_and_process_zap_reply ())
+                return -1;
         } else if (!options.zap_enforce_domain) {
             //  This supports the Stonehouse pattern (encryption without
             //  authentication) in legacy mode (domain set but no handler).
@@ -420,21 +432,15 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
         } else {
             session->get_socket ()->event_handshake_failed_no_detail (
               session->get_endpoint (), EFAULT);
-            rc = -1;
-            goto exit;
+            return -1;
         }
     } else {
         //  This supports the Stonehouse pattern (encryption without authentication).
         state = sending_ready;
     }
 
-    rc = parse_metadata (initiate_plaintext + crypto_box_ZEROBYTES + 128,
-                         clen - crypto_box_ZEROBYTES - 128);
-
-exit:
-    free (initiate_plaintext);
-    free (initiate_box);
-    return rc;
+    return parse_metadata (&initiate_plaintext[crypto_box_ZEROBYTES + 128],
+                           clen - crypto_box_ZEROBYTES - 128);
 }
 
 int zmq::curve_server_t::produce_ready (msg_t *msg_)
@@ -442,30 +448,27 @@ int zmq::curve_server_t::produce_ready (msg_t *msg_)
     const size_t metadata_length = basic_properties_len ();
     uint8_t ready_nonce[crypto_box_NONCEBYTES];
 
-    uint8_t *ready_plaintext =
-      static_cast<uint8_t *> (malloc (crypto_box_ZEROBYTES + metadata_length));
-    alloc_assert (ready_plaintext);
+    std::vector<uint8_t, secure_allocator_t<uint8_t> > ready_plaintext (
+      crypto_box_ZEROBYTES + metadata_length);
 
     //  Create Box [metadata](S'->C')
-    memset (ready_plaintext, 0, crypto_box_ZEROBYTES);
-    uint8_t *ptr = ready_plaintext + crypto_box_ZEROBYTES;
+    std::fill (ready_plaintext.begin (),
+               ready_plaintext.begin () + crypto_box_ZEROBYTES, 0);
+    uint8_t *ptr = &ready_plaintext[crypto_box_ZEROBYTES];
 
     ptr += add_basic_properties (ptr, metadata_length);
-    const size_t mlen = ptr - ready_plaintext;
+    const size_t mlen = ptr - &ready_plaintext[0];
 
     memcpy (ready_nonce, "CurveZMQREADY---", 16);
-    put_uint64 (ready_nonce + 16, cn_nonce);
+    put_uint64 (ready_nonce + 16, get_and_inc_nonce ());
 
-    uint8_t *ready_box = static_cast<uint8_t *> (
-      malloc (crypto_box_BOXZEROBYTES + 16 + metadata_length));
-    alloc_assert (ready_box);
+    std::vector<uint8_t> ready_box (crypto_box_BOXZEROBYTES + 16
+                                    + metadata_length);
 
-    int rc = crypto_box_afternm (ready_box, ready_plaintext, mlen, ready_nonce,
-                                 cn_precom);
+    int rc = crypto_box_afternm (&ready_box[0], &ready_plaintext[0], mlen,
+                                 ready_nonce, get_precom_buffer ());
     zmq_assert (rc == 0);
 
-    free (ready_plaintext);
-
     rc = msg_->init_size (14 + mlen - crypto_box_BOXZEROBYTES);
     errno_assert (rc == 0);
 
@@ -475,11 +478,8 @@ int zmq::curve_server_t::produce_ready (msg_t *msg_)
     //  Short nonce, prefixed by "CurveZMQREADY---"
     memcpy (ready + 6, ready_nonce + 16, 8);
     //  Box [metadata](S'->C')
-    memcpy (ready + 14, ready_box + crypto_box_BOXZEROBYTES,
+    memcpy (ready + 14, &ready_box[crypto_box_BOXZEROBYTES],
             mlen - crypto_box_BOXZEROBYTES);
-    free (ready_box);
-
-    cn_nonce++;
 
     return 0;
 }
