@@ -47,7 +47,7 @@
 #include <sys/ucred.h>
 #endif
 
-#if __cplusplus >= 201103L
+#if __cplusplus >= 201103L || (defined _MSC_VER && _MSC_VER >= 1700)
 #include <type_traits>
 #endif
 
@@ -100,6 +100,9 @@ struct options_t
     // Type of service (containing DSCP and ECN socket options)
     int tos;
 
+    // Protocol-defined priority
+    int priority;
+
     //  Socket type.
     int8_t type;
 
@@ -116,6 +119,10 @@ struct options_t
     //  Default 0 (unused)
     int tcp_maxrt;
 
+    //  Disable reconnect under certain conditions
+    //  Default 0
+    int reconnect_stop;
+
     //  Minimum interval between attempts to reconnect, in milliseconds.
     //  Default 100ms
     int reconnect_ivl;
@@ -286,6 +293,21 @@ struct options_t
 
     // Version of monitor events to emit
     int monitor_event_version;
+
+    //  WSS Keys
+    std::string wss_key_pem;
+    std::string wss_cert_pem;
+    std::string wss_trust_pem;
+    std::string wss_hostname;
+    bool wss_trust_system;
+
+    //  Hello msg
+    std::vector<unsigned char> hello_msg;
+    bool can_send_hello_msg;
+
+    //  Disconnect msg
+    std::vector<unsigned char> disconnect_msg;
+    bool can_recv_disconnect_msg;
 };
 
 inline bool get_effective_conflate_option (const options_t &options)
@@ -297,10 +319,10 @@ inline bool get_effective_conflate_option (const options_t &options)
                || options.type == ZMQ_SUB);
 }
 
-int do_getsockopt (void *const optval_,
-                   size_t *const optvallen_,
+int do_getsockopt (void *optval_,
+                   size_t *optvallen_,
                    const void *value_,
-                   const size_t value_len_);
+                   size_t value_len_);
 
 template <typename T>
 int do_getsockopt (void *const optval_, size_t *const optvallen_, T value_)
@@ -312,17 +334,17 @@ int do_getsockopt (void *const optval_, size_t *const optvallen_, T value_)
     return do_getsockopt (optval_, optvallen_, &value_, sizeof (T));
 }
 
-int do_getsockopt (void *const optval_,
-                   size_t *const optvallen_,
+int do_getsockopt (void *optval_,
+                   size_t *optvallen_,
                    const std::string &value_);
 
-int do_setsockopt_int_as_bool_strict (const void *const optval_,
-                                      const size_t optvallen_,
-                                      bool *const out_value_);
+int do_setsockopt_int_as_bool_strict (const void *optval_,
+                                      size_t optvallen_,
+                                      bool *out_value_);
 
-int do_setsockopt_int_as_bool_relaxed (const void *const optval_,
-                                       const size_t optvallen_,
-                                       bool *const out_value_);
+int do_setsockopt_int_as_bool_relaxed (const void *optval_,
+                                       size_t optvallen_,
+                                       bool *out_value_);
 }
 
 #endif
