@@ -37,10 +37,11 @@
 #include "array.hpp"
 #include "blob.hpp"
 #include "options.hpp"
+#include "endpoint.hpp"
+#include "msg.hpp"
 
 namespace zmq
 {
-class msg_t;
 class pipe_t;
 
 //  Create a pipepair for bi-directional transfer of messages.
@@ -53,12 +54,12 @@ class pipe_t;
 //  read (older messages are discarded)
 int pipepair (zmq::object_t *parents_[2],
               zmq::pipe_t *pipes_[2],
-              int hwms_[2],
-              bool conflate_[2]);
+              const int hwms_[2],
+              const bool conflate_[2]);
 
 struct i_pipe_events
 {
-    virtual ~i_pipe_events () {}
+    virtual ~i_pipe_events () ZMQ_DEFAULT;
 
     virtual void read_activated (zmq::pipe_t *pipe_) = 0;
     virtual void write_activated (zmq::pipe_t *pipe_) = 0;
@@ -70,16 +71,16 @@ struct i_pipe_events
 //  The array of inbound pipes (1), the array of outbound pipes (2) and
 //  the generic array of pipes to be deallocated (3).
 
-class pipe_t : public object_t,
-               public array_item_t<1>,
-               public array_item_t<2>,
-               public array_item_t<3>
+class pipe_t ZMQ_FINAL : public object_t,
+                         public array_item_t<1>,
+                         public array_item_t<2>,
+                         public array_item_t<3>
 {
     //  This allows pipepair to create pipe objects.
     friend int pipepair (zmq::object_t *parents_[2],
                          zmq::pipe_t *pipes_[2],
-                         int hwms_[2],
-                         bool conflate_[2]);
+                         const int hwms_[2],
+                         const bool conflate_[2]);
 
   public:
     //  Specifies the object to send events to.
@@ -107,7 +108,7 @@ class pipe_t : public object_t,
     //  Writes a message to the underlying pipe. Returns false if the
     //  message does not pass check_write. If false, the message object
     //  retains ownership of its message buffer.
-    bool write (msg_t *msg_);
+    bool write (const msg_t *msg_);
 
     //  Remove unfinished parts of the outbound message from the pipe.
     void rollback () const;
@@ -141,20 +142,29 @@ class pipe_t : public object_t,
     //  Returns true if HWM is not reached
     bool check_hwm () const;
 
-    void set_endpoint_uri (const char *name_);
-    std::string &get_endpoint_uri ();
+    void set_endpoint_pair (endpoint_uri_pair_t endpoint_pair_);
+    const endpoint_uri_pair_t &get_endpoint_pair () const;
+
+    void send_stats_to_peer (own_t *socket_base_);
+
+    void send_disconnect_msg ();
+    void set_disconnect_msg (const std::vector<unsigned char> &disconnect_);
 
   private:
     //  Type of the underlying lock-free pipe.
     typedef ypipe_base_t<msg_t> upipe_t;
 
     //  Command handlers.
-    void process_activate_read ();
-    void process_activate_write (uint64_t msgs_read_);
-    void process_hiccup (void *pipe_);
-    void process_pipe_term ();
-    void process_pipe_term_ack ();
-    void process_pipe_hwm (int inhwm_, int outhwm_);
+    void process_activate_read () ZMQ_OVERRIDE;
+    void process_activate_write (uint64_t msgs_read_) ZMQ_OVERRIDE;
+    void process_hiccup (void *pipe_) ZMQ_OVERRIDE;
+    void
+    process_pipe_peer_stats (uint64_t queue_count_,
+                             own_t *socket_base_,
+                             endpoint_uri_pair_t *endpoint_pair_) ZMQ_OVERRIDE;
+    void process_pipe_term () ZMQ_OVERRIDE;
+    void process_pipe_term_ack () ZMQ_OVERRIDE;
+    void process_pipe_hwm (int inhwm_, int outhwm_) ZMQ_OVERRIDE;
 
     //  Handler for delimiter read from the pipe.
     void process_delimiter ();
@@ -173,7 +183,7 @@ class pipe_t : public object_t,
     void set_peer (pipe_t *peer_);
 
     //  Destructor is private. Pipe objects destroy themselves.
-    ~pipe_t ();
+    ~pipe_t () ZMQ_OVERRIDE;
 
     //  Underlying pipes for both directions.
     upipe_t *_in_pipe;
@@ -247,16 +257,18 @@ class pipe_t : public object_t,
 
     const bool _conflate;
 
-    // If the pipe belongs to socket's endpoint the endpoint's name is stored here.
-    // Otherwise this is empty.
-    std::string _endpoint_uri;
+    // The endpoints of this pipe.
+    endpoint_uri_pair_t _endpoint_pair;
 
-    //  Disable copying.
-    pipe_t (const pipe_t &);
-    const pipe_t &operator= (const pipe_t &);
+    // Disconnect msg
+    msg_t _disconnect_msg;
+
+    ZMQ_NON_COPYABLE_NOR_MOVABLE (pipe_t)
 };
 
 void send_routing_id (pipe_t *pipe_, const options_t &options_);
+
+void send_hello_msg (pipe_t *pipe_, const options_t &options_);
 }
 
 #endif
