@@ -35,6 +35,7 @@
 #include "pipe.hpp"
 #include "likely.hpp"
 #include "tcp_connecter.hpp"
+#include "ws_connecter.hpp"
 #include "ipc_connecter.hpp"
 #include "tipc_connecter.hpp"
 #include "socks_connecter.hpp"
@@ -86,9 +87,23 @@ zmq::session_base_t *zmq::session_base_t::create (class io_thread_t *io_thread_,
         case ZMQ_GATHER:
         case ZMQ_SCATTER:
         case ZMQ_DGRAM:
+        case ZMQ_PEER:
+        case ZMQ_CHANNEL:
+#ifdef ZMQ_BUILD_DRAFT_API
+            if (options_.can_send_hello_msg && options_.hello_msg.size () > 0)
+                s = new (std::nothrow) hello_msg_session_t (
+                  io_thread_, active_, socket_, options_, addr_);
+            else
+                s = new (std::nothrow) session_base_t (
+                  io_thread_, active_, socket_, options_, addr_);
+
+            break;
+#else
             s = new (std::nothrow)
               session_base_t (io_thread_, active_, socket_, options_, addr_);
             break;
+#endif
+
         default:
             errno = EINVAL;
             return NULL;
@@ -114,10 +129,14 @@ zmq::session_base_t::session_base_t (class io_thread_t *io_thread_,
     _io_thread (io_thread_),
     _has_linger_timer (false),
     _addr (addr_)
+#ifdef ZMQ_HAVE_WSS
+    ,
+    _wss_hostname (options_.wss_hostname)
+#endif
 {
 }
 
-const char *zmq::session_base_t::get_endpoint () const
+const zmq::endpoint_uri_pair_t &zmq::session_base_t::get_endpoint () const
 {
     return _engine->get_endpoint ();
 }
@@ -168,7 +187,7 @@ int zmq::session_base_t::push_msg (msg_t *msg_)
         && !msg_->is_cancel ())
         return 0;
     if (_pipe && _pipe->write (msg_)) {
-        int rc = msg_->init ();
+        const int rc = msg_->init ();
         errno_assert (rc == 0);
         return 0;
     }
@@ -289,7 +308,8 @@ void zmq::session_base_t::read_activated (pipe_t *pipe_)
     }
 
     if (unlikely (_engine == NULL)) {
-        _pipe->check_read ();
+        if (_pipe)
+            _pipe->check_read ();
         return;
     }
 
@@ -320,7 +340,7 @@ void zmq::session_base_t::hiccuped (pipe_t *)
     zmq_assert (false);
 }
 
-zmq::socket_base_t *zmq::session_base_t::get_socket ()
+zmq::socket_base_t *zmq::session_base_t::get_socket () const
 {
     return _socket;
 }
@@ -380,7 +400,7 @@ int zmq::session_base_t::zap_connect ()
     return 0;
 }
 
-bool zmq::session_base_t::zap_enabled ()
+bool zmq::session_base_t::zap_enabled () const
 {
     return (options.mechanism != ZMQ_NULL || !options.zap_domain.empty ());
 }
@@ -388,7 +408,18 @@ bool zmq::session_base_t::zap_enabled ()
 void zmq::session_base_t::process_attach (i_engine *engine_)
 {
     zmq_assert (engine_ != NULL);
+    zmq_assert (!_engine);
+    _engine = engine_;
 
+    if (!engine_->has_handshake_stage ())
+        engine_ready ();
+
+    //  Plug in the engine.
+    _engine->plug (_io_thread, this);
+}
+
+void zmq::session_base_t::engine_ready ()
+{
     //  Create the pipe if it does not exist yet.
     if (!_pipe && !is_terminating ()) {
         object_t *parents[2] = {this, _socket};
@@ -399,7 +430,7 @@ void zmq::session_base_t::process_attach (i_engine *engine_)
         int hwms[2] = {conflate ? -1 : options.rcvhwm,
                        conflate ? -1 : options.sndhwm};
         bool conflates[2] = {conflate, conflate};
-        int rc = pipepair (parents, pipes, hwms, conflates);
+        const int rc = pipepair (parents, pipes, hwms, conflates);
         errno_assert (rc == 0);
 
         //  Plug the local end of the pipe.
@@ -409,40 +440,50 @@ void zmq::session_base_t::process_attach (i_engine *engine_)
         zmq_assert (!_pipe);
         _pipe = pipes[0];
 
+        //  The endpoints strings are not set on bind, set them here so that
+        //  events can use them.
+        pipes[0]->set_endpoint_pair (_engine->get_endpoint ());
+        pipes[1]->set_endpoint_pair (_engine->get_endpoint ());
+
         //  Ask socket to plug into the remote end of the pipe.
         send_bind (_socket, pipes[1]);
     }
-
-    //  Plug in the engine.
-    zmq_assert (!_engine);
-    _engine = engine_;
-    _engine->plug (_io_thread, this);
 }
 
-void zmq::session_base_t::engine_error (
-  zmq::stream_engine_t::error_reason_t reason_)
+void zmq::session_base_t::engine_error (bool handshaked_,
+                                        zmq::i_engine::error_reason_t reason_)
 {
     //  Engine is dead. Let's forget about it.
     _engine = NULL;
 
     //  Remove any half-done messages from the pipes.
-    if (_pipe)
+    if (_pipe) {
         clean_pipes ();
 
-    zmq_assert (reason_ == stream_engine_t::connection_error
-                || reason_ == stream_engine_t::timeout_error
-                || reason_ == stream_engine_t::protocol_error);
+#ifdef ZMQ_BUILD_DRAFT_API
+        //  Only send disconnect message if socket was accepted and handshake was completed
+        if (!_active && handshaked_ && options.can_recv_disconnect_msg
+            && !options.disconnect_msg.empty ()) {
+            _pipe->set_disconnect_msg (options.disconnect_msg);
+            _pipe->send_disconnect_msg ();
+        }
+#endif
+    }
+
+    zmq_assert (reason_ == i_engine::connection_error
+                || reason_ == i_engine::timeout_error
+                || reason_ == i_engine::protocol_error);
 
     switch (reason_) {
-        case stream_engine_t::timeout_error:
+        case i_engine::timeout_error:
             /* FALLTHROUGH */
-        case stream_engine_t::connection_error:
+        case i_engine::connection_error:
             if (_active) {
                 reconnect ();
                 break;
             }
-            /* FALLTHROUGH */
-        case stream_engine_t::protocol_error:
+
+        case i_engine::protocol_error:
             if (_pending) {
                 if (_pipe)
                     _pipe->terminate (false);
@@ -513,12 +554,25 @@ void zmq::session_base_t::timer_event (int id_)
     _pipe->terminate (false);
 }
 
+void zmq::session_base_t::process_conn_failed ()
+{
+    std::string *ep = new (std::string);
+    _addr->to_string (*ep);
+    send_term_endpoint (_socket, ep);
+}
+
 void zmq::session_base_t::reconnect ()
 {
     //  For delayed connect situations, terminate the pipe
     //  and reestablish later on
-    if (_pipe && options.immediate == 1 && _addr->protocol != "pgm"
-        && _addr->protocol != "epgm" && _addr->protocol != "norm"
+    if (_pipe && options.immediate == 1
+#ifdef ZMQ_HAVE_OPENPGM
+        && _addr->protocol != protocol_name::pgm
+        && _addr->protocol != protocol_name::epgm
+#endif
+#ifdef ZMQ_HAVE_NORM
+        && _addr->protocol != protocol_name::norm
+#endif
         && _addr->protocol != protocol_name::udp) {
         _pipe->hiccup ();
         _pipe->terminate (false);
@@ -534,7 +588,7 @@ void zmq::session_base_t::reconnect ()
     reset ();
 
     //  Reconnect.
-    if (options.reconnect_ivl != -1)
+    if (options.reconnect_ivl > 0)
         start_connecting (true);
     else {
         std::string *ep = new (std::string);
@@ -550,54 +604,6 @@ void zmq::session_base_t::reconnect ()
         _pipe->hiccup ();
 }
 
-zmq::session_base_t::connecter_factory_entry_t
-  zmq::session_base_t::_connecter_factories[] = {
-    connecter_factory_entry_t (protocol_name::tcp,
-                               &zmq::session_base_t::create_connecter_tcp),
-#if !defined ZMQ_HAVE_WINDOWS && !defined ZMQ_HAVE_OPENVMS                     \
-  && !defined ZMQ_HAVE_VXWORKS
-    connecter_factory_entry_t (protocol_name::ipc,
-                               &zmq::session_base_t::create_connecter_ipc),
-#endif
-#if defined ZMQ_HAVE_TIPC
-    connecter_factory_entry_t (protocol_name::tipc,
-                               &zmq::session_base_t::create_connecter_tipc),
-#endif
-#if defined ZMQ_HAVE_VMCI
-    connecter_factory_entry_t (protocol_name::vmci,
-                               &zmq::session_base_t::create_connecter_vmci),
-#endif
-};
-
-zmq::session_base_t::connecter_factory_map_t
-  zmq::session_base_t::_connecter_factories_map (
-    _connecter_factories,
-    _connecter_factories
-      + sizeof (_connecter_factories) / sizeof (_connecter_factories[0]));
-
-zmq::session_base_t::start_connecting_entry_t
-  zmq::session_base_t::_start_connecting_entries[] = {
-    start_connecting_entry_t (protocol_name::udp,
-                              &zmq::session_base_t::start_connecting_udp),
-#if defined ZMQ_HAVE_OPENPGM
-    start_connecting_entry_t ("pgm",
-                              &zmq::session_base_t::start_connecting_pgm),
-    start_connecting_entry_t ("epgm",
-                              &zmq::session_base_t::start_connecting_pgm),
-#endif
-#if defined ZMQ_HAVE_NORM
-    start_connecting_entry_t ("norm",
-                              &zmq::session_base_t::start_connecting_norm),
-#endif
-};
-
-zmq::session_base_t::start_connecting_map_t
-  zmq::session_base_t::_start_connecting_map (
-    _start_connecting_entries,
-    _start_connecting_entries
-      + sizeof (_start_connecting_entries)
-          / sizeof (_start_connecting_entries[0]));
-
 void zmq::session_base_t::start_connecting (bool wait_)
 {
     zmq_assert (_active);
@@ -608,160 +614,198 @@ void zmq::session_base_t::start_connecting (bool wait_)
     zmq_assert (io_thread);
 
     //  Create the connecter object.
-    const connecter_factory_map_t::const_iterator connecter_factories_it =
-      _connecter_factories_map.find (_addr->protocol);
-    if (connecter_factories_it != _connecter_factories_map.end ()) {
-        own_t *connecter =
-          (this->*connecter_factories_it->second) (io_thread, wait_);
-
+    own_t *connecter = NULL;
+    if (_addr->protocol == protocol_name::tcp) {
+        if (!options.socks_proxy_address.empty ()) {
+            address_t *proxy_address = new (std::nothrow)
+              address_t (protocol_name::tcp, options.socks_proxy_address,
+                         this->get_ctx ());
+            alloc_assert (proxy_address);
+            connecter = new (std::nothrow) socks_connecter_t (
+              io_thread, this, options, _addr, proxy_address, wait_);
+            alloc_assert (connecter);
+            if (!options.socks_proxy_username.empty ()) {
+                reinterpret_cast<socks_connecter_t *> (connecter)
+                  ->set_auth_method_basic (options.socks_proxy_username,
+                                           options.socks_proxy_password);
+            }
+        } else {
+            connecter = new (std::nothrow)
+              tcp_connecter_t (io_thread, this, options, _addr, wait_);
+        }
+    }
+#if defined ZMQ_HAVE_IPC
+    else if (_addr->protocol == protocol_name::ipc) {
+        connecter = new (std::nothrow)
+          ipc_connecter_t (io_thread, this, options, _addr, wait_);
+    }
+#endif
+#if defined ZMQ_HAVE_TIPC
+    else if (_addr->protocol == protocol_name::tipc) {
+        connecter = new (std::nothrow)
+          tipc_connecter_t (io_thread, this, options, _addr, wait_);
+    }
+#endif
+#if defined ZMQ_HAVE_VMCI
+    else if (_addr->protocol == protocol_name::vmci) {
+        connecter = new (std::nothrow)
+          vmci_connecter_t (io_thread, this, options, _addr, wait_);
+    }
+#endif
+#if defined ZMQ_HAVE_WS
+    else if (_addr->protocol == protocol_name::ws) {
+        connecter = new (std::nothrow) ws_connecter_t (
+          io_thread, this, options, _addr, wait_, false, std::string ());
+    }
+#endif
+#if defined ZMQ_HAVE_WSS
+    else if (_addr->protocol == protocol_name::wss) {
+        connecter = new (std::nothrow) ws_connecter_t (
+          io_thread, this, options, _addr, wait_, true, _wss_hostname);
+    }
+#endif
+    if (connecter != NULL) {
         alloc_assert (connecter);
         launch_child (connecter);
         return;
     }
-    const start_connecting_map_t::const_iterator start_connecting_it =
-      _start_connecting_map.find (_addr->protocol);
-    if (start_connecting_it != _start_connecting_map.end ()) {
-        (this->*start_connecting_it->second) (io_thread);
+
+    if (_addr->protocol == protocol_name::udp) {
+        zmq_assert (options.type == ZMQ_DISH || options.type == ZMQ_RADIO
+                    || options.type == ZMQ_DGRAM);
+
+        udp_engine_t *engine = new (std::nothrow) udp_engine_t (options);
+        alloc_assert (engine);
+
+        bool recv = false;
+        bool send = false;
+
+        if (options.type == ZMQ_RADIO) {
+            send = true;
+            recv = false;
+        } else if (options.type == ZMQ_DISH) {
+            send = false;
+            recv = true;
+        } else if (options.type == ZMQ_DGRAM) {
+            send = true;
+            recv = true;
+        }
+
+        int rc = engine->init (_addr, send, recv);
+        errno_assert (rc == 0);
+
+        send_attach (this, engine);
+
         return;
     }
 
+#ifdef ZMQ_HAVE_OPENPGM
+
+    //  Both PGM and EPGM transports are using the same infrastructure.
+    if (_addr->protocol == "pgm" || _addr->protocol == "epgm") {
+        zmq_assert (options.type == ZMQ_PUB || options.type == ZMQ_XPUB
+                    || options.type == ZMQ_SUB || options.type == ZMQ_XSUB);
+
+        //  For EPGM transport with UDP encapsulation of PGM is used.
+        bool const udp_encapsulation = _addr->protocol == "epgm";
+
+        //  At this point we'll create message pipes to the session straight
+        //  away. There's no point in delaying it as no concept of 'connect'
+        //  exists with PGM anyway.
+        if (options.type == ZMQ_PUB || options.type == ZMQ_XPUB) {
+            //  PGM sender.
+            pgm_sender_t *pgm_sender =
+              new (std::nothrow) pgm_sender_t (io_thread, options);
+            alloc_assert (pgm_sender);
+
+            int rc =
+              pgm_sender->init (udp_encapsulation, _addr->address.c_str ());
+            errno_assert (rc == 0);
+
+            send_attach (this, pgm_sender);
+        } else {
+            //  PGM receiver.
+            pgm_receiver_t *pgm_receiver =
+              new (std::nothrow) pgm_receiver_t (io_thread, options);
+            alloc_assert (pgm_receiver);
+
+            int rc =
+              pgm_receiver->init (udp_encapsulation, _addr->address.c_str ());
+            errno_assert (rc == 0);
+
+            send_attach (this, pgm_receiver);
+        }
+
+        return;
+    }
+#endif
+
+#ifdef ZMQ_HAVE_NORM
+    if (_addr->protocol == "norm") {
+        //  At this point we'll create message pipes to the session straight
+        //  away. There's no point in delaying it as no concept of 'connect'
+        //  exists with NORM anyway.
+        if (options.type == ZMQ_PUB || options.type == ZMQ_XPUB) {
+            //  NORM sender.
+            norm_engine_t *norm_sender =
+              new (std::nothrow) norm_engine_t (io_thread, options);
+            alloc_assert (norm_sender);
+
+            int rc = norm_sender->init (_addr->address.c_str (), true, false);
+            errno_assert (rc == 0);
+
+            send_attach (this, norm_sender);
+        } else { // ZMQ_SUB or ZMQ_XSUB
+
+            //  NORM receiver.
+            norm_engine_t *norm_receiver =
+              new (std::nothrow) norm_engine_t (io_thread, options);
+            alloc_assert (norm_receiver);
+
+            int rc = norm_receiver->init (_addr->address.c_str (), false, true);
+            errno_assert (rc == 0);
+
+            send_attach (this, norm_receiver);
+        }
+        return;
+    }
+#endif // ZMQ_HAVE_NORM
+
     zmq_assert (false);
 }
 
-#if defined ZMQ_HAVE_VMCI
-zmq::own_t *zmq::session_base_t::create_connecter_vmci (io_thread_t *io_thread_,
-                                                        bool wait_)
+zmq::hello_msg_session_t::hello_msg_session_t (io_thread_t *io_thread_,
+                                               bool connect_,
+                                               socket_base_t *socket_,
+                                               const options_t &options_,
+                                               address_t *addr_) :
+    session_base_t (io_thread_, connect_, socket_, options_, addr_),
+    _new_pipe (true)
 {
-    return new (std::nothrow)
-      vmci_connecter_t (io_thread_, this, options, _addr, wait_);
-}
-#endif
-
-#if defined ZMQ_HAVE_TIPC
-zmq::own_t *zmq::session_base_t::create_connecter_tipc (io_thread_t *io_thread_,
-                                                        bool wait_)
-{
-    return new (std::nothrow)
-      tipc_connecter_t (io_thread_, this, options, _addr, wait_);
-}
-#endif
-
-#if !defined ZMQ_HAVE_WINDOWS && !defined ZMQ_HAVE_OPENVMS                     \
-  && !defined ZMQ_HAVE_VXWORKS
-zmq::own_t *zmq::session_base_t::create_connecter_ipc (io_thread_t *io_thread_,
-                                                       bool wait_)
-{
-    return new (std::nothrow)
-      ipc_connecter_t (io_thread_, this, options, _addr, wait_);
-}
-#endif
-
-zmq::own_t *zmq::session_base_t::create_connecter_tcp (io_thread_t *io_thread_,
-                                                       bool wait_)
-{
-    if (!options.socks_proxy_address.empty ()) {
-        address_t *proxy_address = new (std::nothrow) address_t (
-          protocol_name::tcp, options.socks_proxy_address, this->get_ctx ());
-        alloc_assert (proxy_address);
-        return new (std::nothrow) socks_connecter_t (
-          io_thread_, this, options, _addr, proxy_address, wait_);
-    }
-    return new (std::nothrow)
-      tcp_connecter_t (io_thread_, this, options, _addr, wait_);
 }
 
-#ifdef ZMQ_HAVE_OPENPGM
-void zmq::session_base_t::start_connecting_pgm (io_thread_t *io_thread_)
+zmq::hello_msg_session_t::~hello_msg_session_t ()
 {
-    zmq_assert (options.type == ZMQ_PUB || options.type == ZMQ_XPUB
-                || options.type == ZMQ_SUB || options.type == ZMQ_XSUB);
+}
 
-    //  For EPGM transport with UDP encapsulation of PGM is used.
-    bool const udp_encapsulation = _addr->protocol == "epgm";
 
-    //  At this point we'll create message pipes to the session straight
-    //  away. There's no point in delaying it as no concept of 'connect'
-    //  exists with PGM anyway.
-    if (options.type == ZMQ_PUB || options.type == ZMQ_XPUB) {
-        //  PGM sender.
-        pgm_sender_t *pgm_sender =
-          new (std::nothrow) pgm_sender_t (io_thread_, options);
-        alloc_assert (pgm_sender);
+int zmq::hello_msg_session_t::pull_msg (msg_t *msg_)
+{
+    if (_new_pipe) {
+        _new_pipe = false;
 
-        int rc = pgm_sender->init (udp_encapsulation, _addr->address.c_str ());
+        const int rc =
+          msg_->init_buffer (&options.hello_msg[0], options.hello_msg.size ());
         errno_assert (rc == 0);
 
-        send_attach (this, pgm_sender);
-    } else {
-        //  PGM receiver.
-        pgm_receiver_t *pgm_receiver =
-          new (std::nothrow) pgm_receiver_t (io_thread_, options);
-        alloc_assert (pgm_receiver);
-
-        int rc =
-          pgm_receiver->init (udp_encapsulation, _addr->address.c_str ());
-        errno_assert (rc == 0);
-
-        send_attach (this, pgm_receiver);
+        return 0;
     }
-}
-#endif
 
-#ifdef ZMQ_HAVE_NORM
-void zmq::session_base_t::start_connecting_norm (io_thread_t *io_thread_)
+    return session_base_t::pull_msg (msg_);
+}
+
+void zmq::hello_msg_session_t::reset ()
 {
-    //  At this point we'll create message pipes to the session straight
-    //  away. There's no point in delaying it as no concept of 'connect'
-    //  exists with NORM anyway.
-    if (options.type == ZMQ_PUB || options.type == ZMQ_XPUB) {
-        //  NORM sender.
-        norm_engine_t *norm_sender =
-          new (std::nothrow) norm_engine_t (io_thread_, options);
-        alloc_assert (norm_sender);
-
-        int rc = norm_sender->init (_addr->address.c_str (), true, false);
-        errno_assert (rc == 0);
-
-        send_attach (this, norm_sender);
-    } else { // ZMQ_SUB or ZMQ_XSUB
-
-        //  NORM receiver.
-        norm_engine_t *norm_receiver =
-          new (std::nothrow) norm_engine_t (io_thread_, options);
-        alloc_assert (norm_receiver);
-
-        int rc = norm_receiver->init (_addr->address.c_str (), false, true);
-        errno_assert (rc == 0);
-
-        send_attach (this, norm_receiver);
-    }
-}
-#endif
-
-void zmq::session_base_t::start_connecting_udp (io_thread_t * /*io_thread_*/)
-{
-    zmq_assert (options.type == ZMQ_DISH || options.type == ZMQ_RADIO
-                || options.type == ZMQ_DGRAM);
-
-    udp_engine_t *engine = new (std::nothrow) udp_engine_t (options);
-    alloc_assert (engine);
-
-    bool recv = false;
-    bool send = false;
-
-    if (options.type == ZMQ_RADIO) {
-        send = true;
-        recv = false;
-    } else if (options.type == ZMQ_DISH) {
-        send = false;
-        recv = true;
-    } else if (options.type == ZMQ_DGRAM) {
-        send = true;
-        recv = true;
-    }
-
-    const int rc = engine->init (_addr, send, recv);
-    errno_assert (rc == 0);
-
-    send_attach (this, engine);
+    session_base_t::reset ();
+    _new_pipe = true;
 }
