@@ -236,10 +236,8 @@ void test_simultaneous_connect_bind_threads ()
 
     // Spawn all threads as simultaneously as possible
     for (unsigned int i = 0; i < no_of_times; ++i) {
-        threads[i * 2 + 0] =
-          zmq_threadstart (&simult_conn, (void *) thr_args[i]);
-        threads[i * 2 + 1] =
-          zmq_threadstart (&simult_bind, (void *) thr_args[i]);
+        threads[i * 2 + 0] = zmq_threadstart (&simult_conn, thr_args[i]);
+        threads[i * 2 + 1] = zmq_threadstart (&simult_bind, thr_args[i]);
     }
 
     // Close all threads
