@@ -32,12 +32,6 @@
 
 #include <string.h>
 
-#ifndef _WIN32
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <unistd.h>
-#endif
-
 SETUP_TEARDOWN_TESTCONTEXT
 
 void test_stream_exceeds_buffer ()
@@ -47,16 +41,7 @@ void test_stream_exceeds_buffer ()
     unsigned char rcvbuf[msgsize];
     char my_endpoint[MAX_SOCKET_STRING];
 
-    int server_sock =
-      TEST_ASSERT_SUCCESS_RAW_ERRNO (socket (AF_INET, SOCK_STREAM, 0));
-    int enable = 1;
-    TEST_ASSERT_SUCCESS_RAW_ERRNO (setsockopt (server_sock, SOL_SOCKET,
-                                               SO_REUSEADDR, (char *) &enable,
-                                               sizeof (enable)));
-    struct sockaddr_in saddr = bind_bsd_socket (server_sock);
-    TEST_ASSERT_SUCCESS_RAW_ERRNO (listen (server_sock, 1));
-
-    sprintf (my_endpoint, "tcp://127.0.0.1:%d", ntohs (saddr.sin_port));
+    int server_sock = bind_socket_resolve_port ("127.0.0.1", "0", my_endpoint);
 
     void *zsock = test_context_socket (ZMQ_STREAM);
     TEST_ASSERT_SUCCESS_ERRNO (zmq_connect (zsock, my_endpoint));
